#include "qclockchange.h"
#include <qpushbutton.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlcdnumber.h>
#include <qspinbox.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* 
 *  Constructs a Example which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
qclockchange::qclockchange( QWidget* parent,  const char* name, WFlags fl )
    : qclockchangebase( parent, name, fl )
{
    connect(changeL, SIGNAL(valueChanged(int)), this, SLOT(show_freq()));
    connect(change2N, SIGNAL(valueChanged(int)), this, SLOT(show_freq()));
    connect(changeA, SIGNAL(valueChanged(int)), this, SLOT(show_freq()));
    connect(changeLCD, SIGNAL(valueChanged(int)), this, SLOT(show_freq()));
    connect(changeVCORE, SIGNAL(valueChanged(int)), this, SLOT(show_vcore()));
    connect(writeCCCR, SIGNAL(clicked()), this, SLOT(write_cccr()));
    connect(writeVCORE, SIGNAL(clicked()), this, SLOT(write_vcore()));

    read_cccr();
    read_vcore();
    show_freq();
    show_vcore();

    setCaption( tr( "qclockchange" ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */

qclockchange::~qclockchange()
{
   // no need to delete child widgets, Qt does it all for us
}

int cpuregister;
int vcoreregister;

char *IntToBin(long v, int len, char *s)
{
  char bin[] = "01";
  s[len--] = 0;
  do {
    s[len] = bin[v % 2];
    v = v / 2;
  } while (len--);
  return s;
}

	
void qclockchange::show_freq()
{
	float freq_system;
	float freq_run;
	float freq_turbo;
	float freq_mem;
	float freq_lcd;

	int data_L;
	float data_2N;
	int data_M;
	int data_K;
	int data_A;
	int data_LCD;

	int cccr;
	char tmp[40];
	
	showLNumber_Bin->display(IntToBin(changeL->value(),5,tmp));
	show2NNumber_Bin->display(IntToBin(change2N->value(),4,tmp));
	showANumber_Bin->display(IntToBin(changeA->value(),1,tmp));
	showLCDNumber_Bin->display(IntToBin(changeLCD->value(),1,tmp));

	if (changeL->value() < 2 )
	  data_L = 2;
	else
	  data_L =  changeL->value(); 

	if (data_L < 11 )
	  data_M = 1;
	else if (data_L < 21 )
	  data_M = 2;
	else
	  data_M = 4;

	if (data_L < 8 )
	  data_K = 1;
	else if (data_L < 17 )
	  data_K = 2;
	else
	  data_K = 4;
	
	if (change2N->value() < 2 )
	  data_2N = 1.0;
	else
	  data_2N = change2N->value() / 2.0 ;

	data_A = (changeA->value() + 1) / 2 ;
	data_LCD = changeLCD->value();

	cccr = (changeLCD->value() << 27) + (changeA->value() << 25) + 
	  (change2N->value() << 7) + changeL->value();

	freq_system = 13 * data_L;
	freq_run = freq_system;
	freq_turbo = 13 * data_L * data_2N;
	if (data_A == 0)
	  freq_mem = 13 * data_L / data_M;
	else
	  freq_mem = 13 * data_L;
	
	if (data_LCD == 1)
	  freq_lcd = 26 * data_L / data_K;
	else
	  freq_lcd = 13 * data_L / data_K;

	sprintf(tmp,"%08X",cccr);
	showCCCR->display(tmp);
	showLNumber->display(data_L);
	show2NNumber->display(data_2N);
	showANumber->display(data_A);
	showLCDNumber->display(data_LCD);

	showSystemFreq->display(freq_system);
	showRunModeFreq->display(freq_run);
	showTurboModeFreq->display(freq_turbo);
	showMemoryFreq->display(freq_mem);
	showLCDFreq->display(freq_lcd);

}

void qclockchange::read_cccr()
{
	FILE *fp;

	fp=fopen("/proc/zaurus/CCCR", "r");
	if( fp )
	  fscanf(fp, "%x", &cpuregister);
	fclose(fp);

	changeL->setValue(cpuregister & 0x001f);
	change2N->setValue((cpuregister >> 7) & 0x000F);
	changeA->setValue((cpuregister >> 25) & 0x0001);
	changeLCD->setValue((cpuregister >> 27) & 0x0001);
}

void qclockchange::write_cccr()
{
  
	FILE *fp;

	sync();

	cpuregister = (changeLCD->value() << 27) + (changeA->value() << 25) +
	  (change2N->value() << 7) + changeL->value();
       	fp=fopen("/proc/zaurus/CCCR", "w");
	if( fp )
	  fprintf(fp, "%08X\n", cpuregister);
	fclose(fp);
	
}

void qclockchange::show_vcore()
{
  float vcore;
  int data_vcore;

  char tmp[40];

  showVCORENumber_Bin->display(IntToBin(changeVCORE->value(), 4, tmp));

  vcore = 0.85 + 0.05 * changeVCORE->value();

  showVCOREVoltage->display(vcore);

}
  
void qclockchange::read_vcore()
{
	FILE *fp;

	fp=fopen("/proc/zaurus/VCORE", "r");
	if( fp )
	  fscanf(fp, "%x", &vcoreregister);
	fclose(fp);

	changeVCORE->setValue((vcoreregister ) & 0x0f);
}

void qclockchange::write_vcore()
{
  
	FILE *fp;

	sync();

	vcoreregister = (changeVCORE->value() & 0x0f);
       	fp=fopen("/proc/zaurus/VCORE", "w");
	if( fp )
	  fprintf(fp, "%02X\n", vcoreregister);
	fclose(fp);
  
}
